/*
	Sudoku generator

	(C) Copyright 2005 by Mogens Kjaer, mk@lemo.dk

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>

extern int nfilled,newnfilled,bestndigits;

extern int displayboard[9][9];

#define RNDDIGIT (random()%9+1)
#define NSET 4

extern int setboard[9][9];
extern int setboardsave[9][9];
extern int beforeboard[9][9];
extern int board[9][9];
extern int nmove,nsol,ntest,running,solved;

static int check(int r, int c, int n)
{
  int i,j;
  for(i=0;i<9;i++)
  { if(setboard[i][c]==n) return 0; if(setboard[r][i]==n) return 0; }
  for(i=0;i<3;i++) for(j=0;j<3;j++)
  { if(setboard[r/3*3+i][c/3*3+j]==n) return 0; }
  return 1;
}

static void move()
{
  int i,j,i1,j1,p,best,bestp,n,r,c,empty;
  nmove++;
  r = -1;
  c = -1;
  best = 999;
  empty = 0;

  for(i=0; i<9; i++)
  {
    for(j=0; j<9; j++)
    {
      if(board[i][j])
      {
	p=0;
	empty++;
      }
      else
      {
	p=1022;
	for(i1=0; i1<9; i1++)
	{
	  p &= ~(1<<board[i][i1]);
	  p &= ~(1<<board[i1][j]);
	}
	for(i1=0;i1<3;i1++)
	for(j1=0;j1<3;j1++)
	  p &= ~(1<<board[i/3*3+i1][j/3*3+j1]);
	n = p;
	n = (0x5555 & n) + (0x5555 & (n>> 1));
        n = (0x3333 & n) + (0x3333 & (n>> 2));
	n = (0x0f0f & n) + (0x0f0f & (n>> 4));
	n = (0x00ff & n) + (0x00ff & (n>> 8));
	if(n<best)
	{
	  best=n;
	  bestp=p;
	  r = i;
	  c = j;
	  if(n==0) return; /* Dead end */
	}
      }
    }
  }
  /* printf("%d %d %d %d\n", best, r+1, c+1, empty);  */

  if(r== -1)
  {
  /*
    for(i=0; i<9; i++)
    {
      for(j=0; j<9; j++) printf(" %d", board[i][j]);
      printf("\n");
    }
    printf("\n");
    */
    nsol++;
    return;
  }
  for(i=1;bestp;i++)
  {
    bestp >>= 1;
    if(1 & bestp)
    {
      board[r][c]=i;
      move();
      if(nsol>1) return;
      board[r][c]=0;
    }
  }
}

void sudoku_run()
{
  int i,j,i1,j1,ndigits;
  int i2,j2,k2;
  int walktype;
  char windowtitle[80];

  ntest++;
  sprintf(windowtitle, "Sudoku Generator. Iteration %d, best: %d", ntest, bestndigits);
  settitle(windowtitle);

  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  setboard[i][j] = 0;

  for(i1=0; i1<NSET; i1++)
  {
    do
    {
      i2=RNDDIGIT-1;
      j2=RNDDIGIT-1;
    } while(setboard[i2][j2]);
    for(i=0; i<9; i++)
    for(j=0; j<9; j++)
    board[i][j] = setboard[i][j];
    k2=RNDDIGIT;
    if(check(i2,j2,k2))
    {
      board[i2][j2] = k2;
      nsol=0;
      move();
      if(nsol>0)
      {
	setboard[i2][j2]=k2;
      }
    }
  }


  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  board[i][j] = setboard[i][j];
  nsol=0;
  move();


  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  setboardsave[i][j] = board[i][j];

  for(walktype=0;walktype<16;walktype++)
  {
    ndigits=81;

    for(i1=0; i1<9; i1++)
    for(j1=0; j1<9; j1++)
    setboard[i1][j1] = setboardsave[i1][j1];

    for(i2=0; i2<9; i2++)
    for(j2=0; j2<9; j2++)
    {
      i=i2;
      j=j2;
      if(walktype&1) i=8-i;
      if(walktype&2) j=8-j;
      if(walktype&4) i=(i+4)%9;
      if(walktype&8) j=(j+4)%9;
      for(i1=0; i1<9; i1++)
      for(j1=0; j1<9; j1++)
      board[i1][j1] = setboard[i1][j1];

      board[i][j] = 0;
      nsol=0;
      move();
      if(nsol==1)
      {
	setboard[i][j] = 0;
	ndigits--;
      }
    }
    if(ndigits<=bestndigits)
    {
      for(i=0; i<9; i++)
      for(j=0; j<9; j++)
      displayboard[i][j] = setboard[i][j];
      bestndigits=ndigits;
      main_update();
    }
    if(ndigits<=nfilled)
    {
      running=0;
      setcreatemenutext("Create");
      sprintf(windowtitle, "Sudoku Generator. %d cells in %d iteration%s", ndigits, ntest, ntest==1?"":"s");
      settitle(windowtitle);

      main_update();
      return;
    }
  }
}

void unsolve_sudoku()
{
  int i,j;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  displayboard[i][j] = beforeboard[i][j];
  main_update();
  solved=0;
  setsolvemenutext("Solve");
}

void solve_sudoku()
{
  int i,j;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  {
    board[i][j] = displayboard[i][j];
    beforeboard[i][j] = displayboard[i][j];
  }
  nsol=1;
  move();
  if(nsol>0)
  {
    for(i=0; i<9; i++)
    for(j=0; j<9; j++)
    displayboard[i][j] = beforeboard[i][j]?board[i][j]:-board[i][j];
    main_update();
    solved=1;
    setsolvemenutext("Unsolve");
  }
}

